# -*- coding: UTF-8 -*-
from Util import Util
import random

class NoFreeSpaceException(Exception):
    pass

class RoomSizeGenerator(object):
    """ Generator of room in dungeon.
    """
    def __init__(self, width, height, config):
        self.x = 1
        self.y = 1
        self.width = width
        self.height = height
        self.room_number = config.get('room_number', 3)
        self.checker = SizeDuplicateChecker()

    def get_room_sizes(self):
        """　Get available sizes for Room.

        :rtype: List of dictionary.

            | x: Potision of room x. Express upper left squares.
            | y: Potision of room y. Express upper left squares.
            | width: Width of room.
            | height: Height of room.
        :return: Return sizes.
        """
        room_sizes = []
        for num in range(self.room_number):
            try:
                size = self._get_new_size()
                room_sizes.append(size)
            except NoFreeSpaceException:
                pass
        return room_sizes

    def _get_new_size(self):
        size = self._get_other_size()
        count = 0
        while self.checker.prove_available_size(size) == False:
            count = count + 1
            size = self._get_other_size()
            # 10000回繰り返しても見つからなければもう空きがないとみなして飛ばす
            if count > 10000:
                raise NoFreeSpaceException()
        return size

    def _get_other_size(self):
        room_number = self.room_number

        # ルーム数１は大部屋
        if room_number == 1:
            x = 2
            y = 2
            width = self.width - 2
            height = self.height - 2

        # ルーム数２は最大幅の半分までが最大幅
        elif room_number == 2:
            #
            width = random.randint(10, self.width / 2 - 1)
            height = random.randint(10, self.height / 2 - 1)
            x = random.randint(self.x + 1, self.x + self.width - width - 1)
            y = random.randint(self.y + 1, self.y + self.height - height - 1)

        # それ以上のルーム数は、切り上げた偶数の半分で最大幅を割ったものが最大幅
        # 例えば MAX 30, 部屋数5 -> 最小: 5、最大: 30 / ( 6 / 2 ) = 10
        else:
            room_number = self.room_number
            if self.room_number % 2 == 1:
                room_number = room_number + 1
            room_number = room_number / 2

            # 一番外の境界部分には作らないようにする
            max_width = self.width / room_number - 1
            max_height = self.height / room_number - 1
            width = random.randint(4, max_width)
            height = random.randint(4, max_height)
            x = random.randint(self.x + 1, self.x + self.width - width - 1)
            y = random.randint(self.y + 1, self.y + self.height - height - 1)

        return {
            'x': x,
            'y': y,
            'width': width,
            'height': height
        }


class SizeDuplicateChecker(object):
    def __init__(self, test_list=None):
        if test_list is None:
            self.sizes = []
        else:
            self.sizes = test_list

    def prove_available_size(self, size):
        # 隣接しないようにサイズを2枠分大きいものとして判定する
        # 部屋同士は最低でも3マスあく
        x = size.get('x') - 1
        y = size.get('y') - 1
        width = size.get('width') + 3
        height = size.get('height') + 3

        if self._is_contain(x, y, width, height):
            return False
        else:
            self.sizes.append({
                'x': x,
                'y': y,
                'width': width,
                'height': height
            })
            return True

    def _is_contain(self, x, y, width, height):
        ax = x
        ay = y
        bx = x + width - 1
        by = y + height - 1

        for size in self.sizes:
            cx = size.get('x')
            cy = size.get('y')
            dx = size.get('width') + cx - 1
            dy = size.get('height') + cy - 1

            if Util.is_colliding_square(ax, ay, bx, by, cx, cy, dx, dy):
                return True
                break
        else:
            return False



# -*- coding: UTF-8 -*-
from Rect import Rect
from Tile import Tile
from Door import Door
import uuid, random

class Room(Rect):
    """ Room in dungeon.
        Once instantiated, it is filled with Tile.
    """
    def __init__(self, x, y, width, height, tmp_kind = None):
        super(Room, self).__init__(x, y, width, height)
        self.id = str(uuid.uuid4())
        self.tiles = []
        self.doors = []
        self.walls = []

        self.has_road = False
        """ Whther the at least one of the road, or not."""

        self.north_road_id = None
        """ The road id on north side. If not exist, None."""

        self.south_road_id = None
        """ The road id on south side. If not exist, None."""

        self.east_road_id = None
        """ The road id on east side. If not exist, None."""

        self.west_road_id = None
        """ The road id on west side. If not exist, None."""

        self._fill_tiles(tmp_kind)
        self._make_door()
        self._wrap_walls()

    def _fill_tiles(self, tmp_kind):
        for col in range(self.x, self.ax + 1):
            for row in range(self.y, self.ay + 1):
                self.tiles.append(Tile(col , row, tmp_kind))

    def _wrap_walls(self):
        # ４隅が重複するが問題ないので気にしなくて良い

        # 上辺
        start = self.x - 1
        end = self.ax + 1
        row = self.y - 1
        for col in range(start, end + 1):
            self.walls.append(Tile(col , row, Tile.WALL))

        # 下辺
        start = self.x - 1
        end = self.ax + 1
        row = self.ay + 1
        for col in range(start, end + 1):
            self.walls.append(Tile(col , row, Tile.WALL))

        # 左辺
        west_door = self.get_west_door()
        start = self.y - 1
        end = self.ay + 1
        col = self.x - 1
        for row in range(start, end + 1):
            self.walls.append(Tile(col , row, Tile.WALL))

        # 右辺
        east_door = self.get_east_door()
        start = self.y - 1
        end = self.ay + 1
        col = self.ax + 1
        for row in range(start, end + 1):
            self.walls.append(Tile(col , row, Tile.WALL))

    def get_tile(self, x, y):
        """ Get a tile from Room.

        :param x: potision x.
        :param y: potision y.
        :rtype: Tile class.
        :return: Return tile. if it does not exist, return None.
        """
        for tile in self.tiles:
            if tile.x == x and tile.y == y:
                return tile
        else:
            return None

    def set_tile(self, x, y, new_tile):
        """ Set a tile to Room.

        :param x: potision x.
        :param y: potision y.
        :param new_tile: new Tile
        """
        for index, tile in enumerate(self.tiles):
            if tile.x == x and tile.y == y:
                self.tiles[index] = new_tile

    def has_tile(self, x, y):
        """ Whether holds the tile.

        :param x: potision x.
        :param y: potision y.
        :rtype: Booean
        :return: Return True, if holds the tile. Otherwise Fasle.
        """
        if (self.x - 1 <= x and x <= self.ax + 1) and \
            (self.y - 1  <= y and y <= self.ay + 1):
            return True
        else:
            return False

    def get_door(self, x, y):
        """ Get a door from Room.

        :param x: potision x.
        :param y: potision y.
        :rtype: Door class.
        :return: Return door. if it does not exist, return None.
        """
        for door in self.doors:
            if door.x == x and door.y == y:
                return door
        else:
            return None

    def has_door(self, door):
        """ Whether holds the door.

        :param x: potision x.
        :param y: potision y.
        :rtype: Booean
        :return: Return True, if holds the tile. Otherwise Fasle.
        """
        if door.kind != Tile.DOOR:
            return False
        for room_door in self.doors:
            if room_door.x == door.x and room_door.y == door.y:
                return True
        else:
            return False

    def get_wall(self, x, y):
        """ Get a wall from Room.

        :param x: potision x.
        :param y: potision y.
        :rtype: Tile class.
        :return: Return tile. if it does not exist, return None.
        """
        for wall in self.walls:
            if wall.x == x and wall.y == y:
                return wall
        else:
            return None

    def _make_door(self):
        #上辺
        dx = random.randint(self.x + 1, self.ax - 1)
        dy = self.y
        door = Door(dx, dy, Door.NORTH)
        self.set_tile(dx, dy, door)
        self.doors.append(door)

        #下辺
        dx = random.randint(self.x + 1, self.ax - 1)
        dy = self.ay
        door = Door(dx, dy, Door.SOUTH)
        self.set_tile(dx, dy, door)
        self.doors.append(door)

        #右辺
        dx = self.ax
        dy = random.randint(self.y + 1, self.ay - 1)
        door = Door(dx, dy, Door.EAST)
        self.set_tile(dx, dy, door)
        self.doors.append(door)

        #左辺
        dx = self.x
        dy = random.randint(self.y + 1, self.ay - 1)
        door = Door(dx, dy, Door.WEST)
        self.set_tile(dx, dy, door)
        self.doors.append(door)

    def get_north_door(self):
        """ Get a door on the north side.

        :rtype: Door
        :return: Return Door.
        """
        for door in self.doors:
            if door.y == self.y:
                return door

    def get_south_door(self):
        """ Get a door on the south side.

        :rtype: Door
        :return: Return Door.
        """
        for door in self.doors:
            if door.y == self.ay:
                return door

    def get_east_door(self):
        """ Get a door on the east side.

        :rtype: Door
        :return: Return Door.
        """
        for door in self.doors:
            if door.x == self.ax:
                return door

    def get_west_door(self):
        """ Get a door on the west side.

        :rtype: Door
        :return: Return Door.
        """
        for door in self.doors:
            if door.x == self.x:
                return door



# -*- coding: UTF-8 -*-
from Rect import Rect
from Tile import Tile
from Room import Room
from Rooms import Rooms
from Road import Road
from Roads import Roads
from Waterway import Waterway
from Util import Util
from RoomSearcher import RoomSearcher
from RoomSizeGenerator import RoomSizeGenerator

class Frame(Rect):
    """ The display frame shape.
        Upper left is the origin.
        The origin starts (1, 1).
    """
    def __init__(self, width, height, config):
        super(Frame, self).__init__(1, 1, width, height)
        self.config = config
        self._pop_rooms()
        self._pop_roads()
        self._pop_waterway()
        self._make_map()

    def _pop_rooms(self):
        self.rooms = Rooms()

        # RoomSizeGeneratorで取得したサイズを元に部屋を作成する
        room_generator = RoomSizeGenerator(self.width, self.height, self.config)
        sizes = room_generator.get_room_sizes()
        for index, size in enumerate(sizes):
            room = Room(size.get('x'), size.get('y'), size.get('width'), size.get('height'))
            self.rooms.add(room)

    def _pop_roads(self):
        self.roads = Roads()

        # RoomSearcherで取得した部屋から部屋への扉の組み合わせから道を作成する
        rooms = self.rooms.get_all()
        searcher = RoomSearcher()
        door_pairs = searcher.get_door_pairs(rooms)
        for doors in door_pairs:
            door1 = doors[0]
            door2 = doors[1]
            road = Road(door1, door2)
            # 部屋に隣接しない道ならば有効
            if not self._is_adjacent(rooms, road):
                self.roads.add(road)

        # 道がひとつもない部屋があれば消す
        if len(self.rooms.get_all()) > 1:
            self.rooms.combine_roads(self.roads.get_all())
            for room in self.rooms.get_all():
                if not room.has_road:
                    self.rooms.delete(room.id)

    def _is_adjacent(self, rooms, road):
        # 道が部屋に隣接してできている場合はTrue、そうでなければFalse
        for room in rooms:
            for tile in road.get_tiles():
                if self._check_adjacent_each_direction(room, tile):
                    return True
        else:
            return False

    def _check_adjacent_each_direction(self, room, tile):
        # 道の１マスの上下左右にドアでも壁でもない部屋のマスがあればTrue、そうでなければFalse
        x = tile.x + 1
        y = tile.y
        if self._is_adjacent_tile(room, x, y):
            return True

        x = tile.x - 1
        y = tile.y
        if self._is_adjacent_tile(room, x, y):
            return True

        x = tile.x
        y = tile.y + 1
        if self._is_adjacent_tile(room, x, y):
            return True

        x = tile.x
        y = tile.y - 1
        if self._is_adjacent_tile(room, x, y):
            return True

        return False

    def _is_adjacent_tile(self, room, x, y):
        if room.has_tile(x, y):
            target_tile = room.get_tile(x, y)
            if target_tile is not None and target_tile.kind != Tile.DOOR and target_tile.kind != Tile.WALL:
                return True
        return False

    def _pop_waterway(self):
        self.waterway = Waterway(self.x, self.y, self.width, self.height, self.config)

    def _make_map(self):
        x = self.x
        y = self.y
        ax = self.ax
        ay = self.ay
        self.dungeon_map = [[0 for j in range(ay)] for i in range(ax)]

        # xとyを順にインクリメントしながら、該当するマスのタイルを取得していく
        floor = ''
        for row in range(x, ax + 1):
            line = ''
            for col in range(y, ay + 1):
                tile = None
                for room in self.rooms.get_all():

                    # 部屋を取得
                    if room.has_tile(col, row):
                        tile = room.get_tile(col, row)

                        # 道を取得（道は壁を通るため先にも見る
                        if tile is None:
                            for road in self.roads.get_all():
                                if road.may_have_tile(col, row):
                                    tile = road.get_tile(col, row)
                                    if tile:
                                        break
                        # 壁を取得
                        if tile is None:
                            tile = room.get_wall(col, row)
                        if tile:
                            break

                # 道を取得
                if tile is None:
                    for road in self.roads.get_all():
                        if road.may_have_tile(col, row):
                            tile = road.get_tile(col, row)
                            if tile:
                                break

                # 水路を取得
                if tile is None:
                    tile = self.waterway.get_tile(col, row)

                if tile:
                    self.dungeon_map[row-1][col-1] = str(tile)
                    line = line + str(tile)
                else:
                    self.dungeon_map[row-1][col-1] = str(' ')
                    line = line + str(' ')
            floor = floor + line + '\n'

        self.dungeon_string = floor

    def to_string(self):
        """Get a human-readable dungeon map.

        :rtype: string
        :return: Fixed-length string with a newline.
        """
        return self.dungeon_string

    def to_array(self):
        """Get a two-dimensional array of dungeon map.

        :rtype: array
        :return: Two-dimensional array.
        """
        return self.dungeon_map



# -*- coding: UTF-8 -*-
class Util(object):

    @staticmethod
    def is_colliding_square(ax, ay, bx, by, cx, cy, dx, dy):
        # abとcdという四角形として
        # cdがののab右にある、cdがabの左にある、cdがabの下にある、cdがabの上にある
        if bx < cx or dx < ax or by < cy or dy < ay:
            # 衝突していない
            return False
        else:
            return True



# -*- coding: UTF-8 -*-

class Roads(object):
    """ Collections of Road.
    """
    def __init__(self):
        self.roads = []

    def add(self, road):
        """Add a road to collection.

        :param tile: tile class.
        """
        self.roads.append(road)

    def get(self, id):
        """Get a road from collection.

        :param id: unique key of the road.
        :rtype: Road class
        :return: Return road. if it does not exist, return None.
        """
        for road in self.roads:
            if road.id == id:
                return road
        else:
            return None

    def get_all(self):
        """Get all roads from collection.

        :rtype: List
        :return: List of Road.
        """
        return self.roads

    def get_copy_all(self):
        """ Get all copy of roads from collection.
            When performing a destructive operation , use this.

        :rtype: List
        :return: List of Road.
        """
        return copy.deepcopy(self.roads)



# -*- coding: UTF-8 -*-
from Frame import Frame
import random

class Generator(object):
    """ Generator two-dimensional map for roguelike.

        Config:

        - row_size: Vertical size of the map.
        - col_size: Horizontal size of the map.
        - room_number: Number of rooms to pop in the map.
    """
    def __init__(self, config={}):
        self.row_size = config.get('col_size', 20)
        self.col_size = config.get('row_size', 20)
        self.dungeon = Frame(self.col_size, self.row_size, config)

    def to_string(self):
        """ Call Frame.to_string()

            See Frame.to_string detail.
        """
        return self.dungeon.to_string()

    def to_array(self):
        """ Call Frame.to_array()

            See Frame.to_array detail.
        """
        return self.dungeon.to_array()



# -*- coding: UTF-8 -*-
import sys, os
sys.path.append(os.path.abspath('./src'))

import math, copy
from operator import attrgetter

class RoomSearcher(object):
    EAST = 'east'
    WEST = 'west'
    NORTH = 'north'
    SOUTH = 'south'

    def get_door_pairs(self, rooms):
        """ Get door pairs from all rooms.

        It is a combination of the door nearest the room from the door of each side of the room.
        :rtype: List of Tupple

            | [0] from_door
            | [1] to_door
        :return: Return List of door tuple.
        """
        self.all_rooms = []
        for room in rooms:
            self.all_rooms.append({
                'id': room.id,
                'x': room.mx,
                'y': room.my
            })

        divided_rooms = self._devide_room()
        combinations_of_nearest_room = self._get_near_conbination(divided_rooms)
        pair_list = self._adjust_result_analyze(combinations_of_nearest_room)
        door_pair = self._transform_door_pair(pair_list, rooms)
        return door_pair

    def _adjust_result_analyze(self, rooms):
        # 一番近い部屋同士の組み合わせを、東西南北ごとに作る
        # 以下の形式を返す
        # [{
        #     'from': room id,
        #     'to': room id,
        #     'direction': direction of north, south, east and west,
        # }]
        self.nearest_rooms = rooms
        road_pair = []
        for nearest_room in rooms:
            from_id = nearest_room.get('id')

            north = nearest_room.get(self.NORTH)
            if north is not None:
                to_id = north.get('id')
                direction = self.NORTH
                road_pair.append({
                    'from': from_id,
                    'to': to_id,
                    'direction': self.NORTH,
                })

            south = nearest_room.get(self.SOUTH)
            if south is not None:
                to_id = south.get('id')
                direction = self.SOUTH
                road_pair.append({
                    'from': from_id,
                    'to': to_id,
                    'direction': self.SOUTH,
                })

            east = nearest_room.get(self.EAST)
            if east is not None:
                to_id = east.get('id')
                direction = self.EAST
                road_pair.append({
                    'from': from_id,
                    'to': to_id,
                    'direction': self.EAST,
                })

            west = nearest_room.get(self.WEST)
            if west is not None:
                to_id = west.get('id')
                direction = self.WEST
                road_pair.append({
                    'from': from_id,
                    'to': to_id,
                    'direction': self.WEST,
                })
        return road_pair

    def _transform_door_pair(self, pair_list, rooms):
        # 部屋の組み合わせからドアの組わせを作る
        # 以下の形式を返す
        # [(
        #    Door, One of the door to be the start of road.
        #    Door  One of the foor to be the end of road.
        # )]
        col_distances = []
        row_distances = []
        door_pair = []
        for pair in pair_list:
            from_id = pair.get('from')
            to_id = pair.get('to')

            for room in rooms:
                if room.id == from_id:
                    from_room = room
                if room.id == to_id:
                    to_room = room

            if pair.get('direction') == RoomSearcher.NORTH:
                door1 = from_room.get_north_door()
                door2 = to_room.get_south_door()
                distance = (door2.x - door1.x) * (door2.x - door1.x) + (door2.y - door1.y) * (door2.y - door1.y)
                # 距離が同じならばドアの組み合わせは同じなので除く
                for way in col_distances:
                    if way == distance:
                        break
                else:
                    col_distances.append(distance)
                    door_pair.append((door1, door2))

            if pair.get('direction') == RoomSearcher.SOUTH:
                door1 = from_room.get_south_door()
                door2 = to_room.get_north_door()
                distance = (door2.x - door1.x) * (door2.x - door1.x) + (door2.y - door1.y) * (door2.y - door1.y)
                # 距離が同じならばドアの組み合わせは同じなので除く
                for way in col_distances:
                    if way == distance:
                        break
                else:
                    col_distances.append(distance)
                    door_pair.append((door1, door2))

            if pair.get('direction') == RoomSearcher.EAST:
                door1 = from_room.get_east_door()
                door2 = to_room.get_west_door()
                distance = (door2.x - door1.x) * (door2.x - door1.x) + (door2.y - door1.y) * (door2.y - door1.y)
                # 距離が同じならばドアの組み合わせは同じなので除く
                for way in row_distances:
                    if way == distance:
                        break
                else:
                    row_distances.append(distance)
                    door_pair.append((door1, door2))

            if pair.get('direction') == RoomSearcher.WEST:
                door1 = from_room.get_west_door()
                door2 = to_room.get_east_door()
                distance = (door2.x - door1.x) * (door2.x - door1.x) + (door2.y - door1.y) * (door2.y - door1.y)
                # 距離が同じならばドアの組み合わせは同じなので除く
                for way in row_distances:
                    if way == distance:
                        break
                else:
                    row_distances.append(distance)
                    door_pair.append((door1, door2))

        return door_pair

    def _devide_room(self):
        # 基準になる部屋とそれ以外の部屋、という組み合わせを作る
        # [(target_room, comparison_destinations[])] を返す
        devided_rooms = []
        for room in self.all_rooms:
            target_room = room
            comparison_destinations = []
            for room in self.all_rooms:
                if room.get('id') != target_room.get('id'):
                    comparison_destinations.append(room)
            devided_rooms.append((target_room, comparison_destinations))
        return devided_rooms

    def _get_near_conbination(self, divided_rooms):
        # 基準になる部屋と東西南北の方向ごとに一番近い部屋の組み合わせのリストを返す
        # 以下の形式を返す
        # {
        #     # Below, see _search_nearest_room detail.
        #     'id':
        #     # Below, see _get_nearest_room detail.
        #     'north':
        #     'south':
        #     'east':
        #     'west'
        # }
        combinations_of_nearest_room = []
        for divided in divided_rooms:
            target_room = divided[0]
            comparison_destinations = divided[1]
            combinations_of_nearest_room.append(
                self._search_nearest_room(target_room, comparison_destinations)
            )
        return combinations_of_nearest_room

    def _search_nearest_room(self, room, destinations):
        # 基準になる部屋から、一番近い部屋を探す
        # 以下の形式を返す。
        # {
        #     'id': ID of standard room,
        #     Other keys, see _get_nearest_room detail.
        # }
        distance_list = self._caluclate_room_distanse(room, destinations)
        result = self._get_nearest_room(distance_list)
        result['id'] = room.get('id')
        return result

    def _get_nearest_room(self, distance_list):
        # 全ての部屋に対する距離の中から、東西南北のそれぞれの方向で一番近い距離を探す
        # 以下の形式を返す。
        # {
        #     'north' : {
        #         'id': Destination room id,
        #         'distance': Distance from room to room,
        #         'angle': Angle of destination room.
        #     } or None,
        #     'south' : {
        #         'id': Destination room id,
        #         'distance': Distance from room to room,
        #         'angle': Angle of destination room.
        #     } or None,
        #     'east' : {
        #         'id': Destination room id,
        #         'distance': Distance from room to room,
        #         'angle': Angle of destination room.
        #     } or None,
        #     'west' : {
        #         'id': Destination room id,
        #         'distance': Distance from room to room,
        #         'angle': Angle of destination room.
        #     } or None,
        # }
        upper_list = []
        right_list = []
        lower_list = []
        left_list = []

        # 角度を元にして四方に振り分ける
        for distance in distance_list:
            angle = math.degrees(distance.get('angle'))
            if -45 <= angle and angle < 45:
                #上
                upper_list.append(distance)
            elif 45 <= angle and angle < 135:
                #右
                right_list.append(distance)
            elif -135 <= angle and angle < -45:
                #左
                left_list.append(distance)
            else:
                #下
                lower_list.append(distance)

        # それぞれのリストの中身をdistanceの値をキーにしてソートする
        # 最も近い部屋までの距離と
        sorted_upper = sorted(upper_list, key=lambda x:x['distance'])
        sorted_right = sorted(right_list, key=lambda x:x['distance'])
        sorted_lower = sorted(lower_list, key=lambda x:x['distance'])
        sorted_left = sorted(left_list, key=lambda x:x['distance'])
        return {
            self.NORTH: self._get_first_item(sorted_upper),
            self.EAST: self._get_first_item(sorted_right),
            self.SOUTH: self._get_first_item(sorted_lower),
            self.WEST: self._get_first_item(sorted_left),
        }

    def _get_first_item(self, list, default=None):
        try:
            return list[0]
        except IndexError:
            return default

    def _caluclate_room_distanse(self, room, destinations):
        # 基準になる部屋から、すべての部屋に対して距離を計算する
        distance_list = []
        ax = room.get('x')
        ay = room.get('y')
        for destination in destinations:
            bx = destination.get('x')
            by = destination.get('y')
            distance = (bx - ax) * (bx - ax) + (by - ay) * (by - ay)

            #マス目は左上原点なので、Y座標が逆になっているものとして計算する。
            #右側面を正の値、左側面を負の値、上方が0度、下方が180/-180度となるものとする。
            radius = math.atan2(bx-ax,by-ay)
            if 0 < radius:
                angle = math.radians(180) - math.atan2(bx-ax,by-ay)
            else:
                angle = math.radians(-180) - math.atan2(bx-ax,by-ay)
            distance_list.append({
                'id': destination.get('id'),
                'distance': distance,
                'angle':angle
            })
        return  distance_list



# -*- coding: UTF-8 -*-
import copy

class Rooms(object):
    """ Collection of room.
    """
    def __init__(self):
        self.rooms = []

    def add(self, room):
        """Add a room to collection.

        :param room: room class.
        """
        self.rooms.append(room)

    def get(self, id):
        """Get a room from collection.

        :param id: unique key of the room.
        :rtype: Room class
        :return: Return room. if it does not exist, return None.
        """
        for room in self.rooms:
            if room.id == id:
                return room
        else:
            return None

    def get_all(self):
        """Get all rooms from collection.

        :rtype: List
        :return: List of Room.
        """
        return self.rooms

    def delete(self, id):
        """ Get a room from collection.

        :param id: unique key of the room.
        """
        for room in self.rooms[:]:
            if room.id == id:
                self.rooms.remove(room)

    def combine_roads(self, roads):
        """ Combine room and road. All room checks.

        When has door, the room records each direction road id.
        :param roads: List of road.
        """
        # 全ての部屋と全ての道を探索し、部屋につながっている道があればその道のIDを設定する
        for room in self.rooms:
            for road in roads:
                if room.has_door(road.from_door) or room.has_door(road.to_door):
                    from_door = road.from_door
                    to_door = road.to_door

                    north_door = room.get_north_door()
                    if (north_door.x == from_door.x and north_door.y == from_door.y) or \
                        (north_door.x == to_door.x and north_door.y == to_door.y):
                        room.has_road = True
                        room.north_road_id = road.id

                    south_door = room.get_south_door()
                    if (south_door.x == from_door.x and south_door.y == from_door.y) or \
                        (south_door.x == to_door.x and south_door.y == to_door.y):
                        room.has_road = True
                        room.south_road_id = road.id

                    east_door = room.get_east_door()
                    if (east_door.x == from_door.x and east_door.y == from_door.y) or \
                        (east_door.x == to_door.x and east_door.y == to_door.y):
                        room.has_road = True
                        room.east_road_id = road.id

                    west_door = room.get_west_door()
                    if (west_door.x == from_door.x and west_door.y == from_door.y) or \
                        (west_door.x == to_door.x and west_door.y == to_door.y):
                        room.has_road = True
                        room.west_road_id = road.id

    def get_copy_all(self):
        """ Get all copy of rooms from collection.
            When performing a destructive operation , use this.

        :rtype: List
        :return: List of Room.
        """
        return copy.deepcopy(self.rooms)

    def __str__(self):
        return str([str(room) for room in self.rooms])



# -*- coding: UTF-8 -*-
from Tile import Tile

class Door(Tile):
    """ A door in Room.
        Door know the direction of way.
    """
    NORTH = 'north'
    SOUTH = 'south'
    EAST = 'east'
    WEST = 'west'
    def __init__(self, x, y, direction):
        super(Door, self).__init__(x, y, kind = Tile.DOOR)
        self.direction = direction

    def is_vertical(self):
        """ Whether the direction of the road or vertical direction.

        :rtype: Boolean
        :return: Return True, the direction is north or south. Otherwise False.
        """
        return self.direction == self.NORTH or self.direction == self.SOUTH

    def is_horizontal(self):
        """ Whether the direction of the road or horizontal direction.

        :rtype: Boolean
        :return: Return True, the direction is east or west. Otherwise False.
        """
        return self.direction == self.EAST or self.direction == self.WEST



# -*- coding: UTF-8 -*-
from Door import Door
from Tile import Tile
import copy, uuid

class Road(object):
    """ Road from door to door.
        Once instantiated, it is filled with Tile.
    """
    def __init__(self, from_door, to_door):
        self.id = str(uuid.uuid4())
        self.tiles = []
        self.from_door = Door(from_door.x, from_door.y, from_door.direction)
        self.to_door = Door(to_door.x, to_door.y, to_door.direction)
        self._fill_road(from_door, to_door)

    def get_tiles(self):
        """Get all tiles from Road.

        :rtype: List of Tile .
        :return: Return tiles. if it does not exist, return empty list.
        """
        return self.tiles

    def get_tile(self, x, y):
        """Get a tile from Road.

        :param x: potision x.
        :param y: potision y.
        :rtype: Tile class.
        :return: Return tile. if it does not exist, return None.
        """
        for tile in self.tiles:
            if tile.x == x and tile.y == y:
                return tile
        else:
            return None

    def set_tile(self, x, y, new_tile):
        """Set a tile to Road.

        :param x: potision x.
        :param y: potision y.
        :param new_tile: new Tile
        """
        for index, tile in enumerate(self.tiles):
            if tile.x == x and tile.y == y:
                self.tiles[index] = new_tile

    def may_have_tile(self, x, y):
        """Whether there is a possibility of holding the tile.

        :param x: potision x.
        :param y: potision y.
        :rtype: Booean
        :return: Return True, if there is a possibility. Otherwise Fasle.
        """
        if (self.x <= x and x <= self.ax) and \
            (self.y <= y and y <= self.ay):
            return True
        else:
            return False

    def _fill_road(self, from_door, to_door):
        # 計算しやすいように、小さい座標の順から大きい座標の場所へ向かうようにする
        # 道が縦向きならY座標を基準に、横向きならX座標を基準にして判定する
        if from_door.is_vertical():
            if from_door.y < to_door.y:
                self._set_inline_range(from_door, to_door, 0, 1)
                self._pave_road(from_door, to_door, 0, 1)
            else:
                self._set_inline_range(to_door, from_door, 0, 1)
                self._pave_road(to_door, from_door, 0, 1)
        else:
            if from_door.x < to_door.x:
                self._set_inline_range(from_door, to_door, 1, 0)
                self._pave_road(from_door, to_door, 1, 0)
            else:
                self._set_inline_range(to_door, from_door, 1, 0)
                self._pave_road(to_door, from_door, 1, 0)

    def _set_inline_range(self, from_door, to_door, col_add, row_add):
        #ドアを含まない道になりうる領域を四角形とみなして保持する
        x = from_door.x + col_add
        y = from_door.y + row_add
        ax = to_door.x - col_add
        ay = to_door.y - row_add

        if x < ax:
            self.x = x
            self.ax = ax
        else:
            self.x = ax
            self.ax = x
        if y < ay:
            self.y = y
            self.ay = ay
        else:
            self.y = ay
            self.ay = y

    def _pave_road(self, from_door, to_door, col_add, row_add):
        # ドアからドアまでの道を作る
        ax = from_door.x + col_add
        ay = from_door.y + row_add
        next1 = Tile(ax, ay, Tile.WAY)
        self.tiles.append(next1)

        if self._pave_middle_polyline(ax, ay, to_door.x, to_door.y, col_add):
            return

        bx = to_door.x - col_add
        by = to_door.y - row_add
        next2 = Tile(bx, by, Tile.WAY)
        self.tiles.append(next2)

        if self._pave_middle_polyline(ax, ay, bx, by, col_add):
            return

        # 道が埋まるまで再帰的に繰り返す
        self._pave_road(next1, next2, col_add, row_add)

    def _pave_middle_polyline(self, ax, ay, bx, by, vertical):
        #縦方向の道は横がそろうまで伸ばす
        if vertical == 0:
            # 同じY位置まできたらX側を埋める
            if ay == by:
                if ax < bx:
                    start = ax
                    end = bx
                else:
                    start = bx
                    end = ax
                for x in range(start, end):
                    road = Tile(x, ay, Tile.WAY)
                    self.tiles.append(road)
                return True

        #横方向の道は横がそろうまで伸ばす
        else:
            # 同じX位置にきたらY側を埋める
            if ax == bx:
                if ay < by:
                    start = ay
                    end = by
                else:
                    start = by
                    end = ay
                for y in range(start, end):
                    road = Tile(ax, y, Tile.WAY)
                    self.tiles.append(road)
                return True

        return False



# -*- coding: UTF-8 -*-

class Tile(object):
    """ A tile of map in dungeon.
        All unit of member value is squares, not coordinate.
    """

    DEFAULT = '_'
    WALL = '+'
    WAY = '#'
    DOOR = '~'
    WATER = '^'

    def __init__(self, x, y, kind = None):
        """
        arguments:
        x -- potision x
        y -- potision y
        kind -- kind of map (Default is 'T')
        """
        self.x = x
        self.y = y
        if kind is None:
            self.kind = self.DEFAULT
        else:
            self.kind = kind

    def __str__(self):
        return str(self.kind)



# -*- coding: UTF-8 -*-

class Rect(object):
    """ Shape of map base.

        All unit of member value is squares, not coordinate.
    """

    def __init__(self, x, y, width, height):
        self.x = x
        """X squares in the lower right."""

        self.y = y
        """Y squares in the lower right."""

        self.width = width
        """Size of width"""

        self.height = height
        """Size of height"""

        self.ax = x + width -1
        """X squares in the lower right. Automatic calculation from the argument"""

        self.ay = y + height -1
        """Y squares in the lower right. Automatic calculation from the argument"""

        self.mx = (self.x + self.ax) / 2
        """X squares midpoint. Automatic calculation from the argument"""

        self.my = (self.y + self.ay) / 2
        """Y squares midpoint. Automatic calculation from the argument"""

    def __str__(self):
        return str({
            'x': self.x,
            'y': self.y,
            'ax': self.ax,
            'ay': self.ay,
            'mx': self.mx,
            'my': self.my,
            'width': self.width,
            'heigth': self.height,
        })



# -*- coding:utf-8 -*-
try:
    import setuptools
    from setuptools import setup, find_packages
except ImportError:
    print("Please install setuptools.")

import sys
import info
import version

setup_options = info.INFO
setup_options["version"] = version.VERSION
setup_options["install_requires"] = open('requirements.txt').read().splitlines()
setup_options["packages"] = ['dungeon']

setup(**setup_options)



# package information.
INFO = dict(
    name = "dungeon-generator",
    description = "Dungeon generator cli tool for rouge-like.",
    author = "Keisuke Ota",
    author_email = "arvelt.s@gmail.com",
    license = "MIT License",
    url = "https://github.com/arvelt/dungeon-generator",
    classifiers = [
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "License :: OSI Approved :: MIT License"
    ]
)



# -*- coding: UTF-8 -*-
from Rect import Rect
from Tile import Tile
import uuid, random, math

class Waterway(Rect):
    """ Waterway in dungeon.

        Randomly pop water tile in dungeon.
        Water to pop divided into pond and tributary.
    """
    def __init__(self, x, y, width, height, config):
        super(Waterway, self).__init__(x, y, width, height)
        self.config = config
        self.tiles = []
        self._pop_water()

    def get_tiles(self):
        """ Get tile collection from Waterway.

        :rtype: List.
        :return: Return tile of list. if it does not exist, return empty list.
        """
        return self.tiles

    def get_tile(self, x, y):
        """ Get a tile from Waterway.

        :param x: potision x.
        :param y: potision y.
        :rtype: Tile class.
        :return: Return tile. if it does not exist, return None.
        """
        for tile in self.tiles:
            if tile.x == x and tile.y == y:
                return tile
        else:
            return None

    def _pop_water(self):
        # いずれの処理もタイルの重複を生むが、問題ないので気にしない
        self._pop_pond()
        self._pop_tributary()

    def _pop_pond(self):
        # 象限ごとにおおまかに目安を決めて、ある程度の大きさの沼を沸かせる

        amount_water = self.config.get('amount_water', 0)
        if amount_water == 0:
            return

        orthant_seed = random.randint(1,4)

        # 第一象限
        if orthant_seed == 1:
            base_col = self.x + self.width / 2
            base_row = self.y
            self._pop_orthant_pond(base_col, base_row)

        # 第二象限
        if orthant_seed == 2:
            base_col = self.x
            base_row = self.y
            self._pop_orthant_pond(base_col, base_row)

        # 第三象限
        if orthant_seed == 3:
            base_col = self.x
            base_row = self.y + self.height / 2
            self._pop_orthant_pond(base_col, base_row)

        # 第四象限
        if orthant_seed == 4:
            base_col = self.x + self.width / 2
            base_row = self.y + self.height / 2
            self._pop_orthant_pond(base_col, base_row)

    def _pop_orthant_pond(self, base_col, base_row):
        # 基準となる範囲をずらして、一定の領域に沼を沸かせる
        col_add = int(math.floor( random.randint(2, self.width / 3) ))
        row_add = int(math.floor( random.randint(2, self.height / 3) ))
        col_adjust = self.width / 3
        row_adjust = self.height / 3

        col = int(math.floor( random.randint(base_col, base_col + col_adjust) ))
        row = int(math.floor( random.randint(base_row, base_row + row_adjust) ))
        for x in range(col, col + col_add):
            for y in range(row, row + row_add):
                self.tiles.append(Tile(x, y, Tile.WATER))

    def _pop_tributary(self):
        # 縦か横かの目安を決めて、時々ずらしながら川を沸かせる
        # config.amount_water が多いほど川は増える amount_waterは最大10

        amount_water = self.config.get('amount_water', 3)
        if amount_water == 0:
            return
        if amount_water > 10:
            amount_water = 10

        _cols = []
        _rows = []

        direction_seed = random.randint(1,3)
        if direction_seed == 1:
            for count in range(amount_water):
                self._pop_horizontal_tributary(_cols)

        if direction_seed == 2:
            for count in range(amount_water):
                self._pop_vertical_tributary(_rows)

        if direction_seed == 3:
            for count in range(amount_water):
                self._pop_horizontal_tributary(_cols)
                self._pop_vertical_tributary(_rows)

    def _pop_horizontal_tributary(self, _cols):
        col = random.randint(self.x, self.ax)
        for _col in _cols:
            if _col == col:
                return
        else:
            for row in range(self.y, self.ay + 1):
                polygonal_seed = random.randint(1, 7)
                if polygonal_seed == 1:
                    col = random.randint(self.x, self.ax)
                self.tiles.append(Tile(col, row, Tile.WATER))

    def _pop_vertical_tributary(self, _rows):
        row = random.randint(self.y, self.ay)
        for _row in _rows:
            if _row == row:
                return
        else:
            for col in range(self.x, self.ax + 1):
                polygonal_seed = random.randint(1, 7)
                if polygonal_seed == 1:
                    row = random.randint(self.y, self.ay)
                self.tiles.append(Tile(col, row, Tile.WATER))
